use axum::{routing::get, Router};
use eframe::egui;
use serde::{Deserialize, Serialize};
use std::{
    net::SocketAddr,
    sync::{Arc, Mutex},
    thread,
};
use tokio::{net::TcpListener, sync::mpsc};

#[derive(Debug, Clone)]
struct AppState {
    settings: Arc<Mutex<ServerSettings>>,
}

#[derive(Debug, Clone, Serialize, Deserialize)]
struct ServerSettings {
    port: String, // Change to String for text editing
}

async fn home() -> &'static str {
    "Hello, World!"
}

#[tokio::main]
async fn main() {
    let (tx, rx) = mpsc::channel(32);
    let settings = Arc::new(Mutex::new(ServerSettings {
        port: "3311".to_string(),
    }));
    let app_state = AppState {
        settings: settings.clone(),
    };

    // Run the server in a separate thread
    let server_thread = thread::spawn(move || {
        let rt = tokio::runtime::Runtime::new().unwrap();
        rt.block_on(async {
            run_server(app_state, rx).await;
        });
    });

    // Run the UI on the main thread
    run_ui(settings, tx);

    // Wait for the server thread to finish
    server_thread.join().unwrap();
}

async fn run_server(app_state: AppState, mut rx: mpsc::Receiver<ServerSettings>) {
    loop {
        let addr = {
            let settings = app_state.settings.lock().unwrap();
            let port: u16 = settings.port.parse().unwrap_or(3311);
            SocketAddr::from(([0, 0, 0, 0], port))
        };

        let app = Router::new()
            .route("/", get(home))
            .with_state(app_state.clone());

        println!("listening on {}", addr);

        let listener = TcpListener::bind(&addr).await.unwrap();
        let server = axum::serve(listener, app.into_make_service());

        server.await.unwrap();
    }
}

fn run_ui(settings: Arc<Mutex<ServerSettings>>, tx: mpsc::Sender<ServerSettings>) {
    let app = NativeApp { settings, tx };
    let native_options = eframe::NativeOptions::default();
    eframe::run_native(
        "Custom window frame", // unused title
        native_options,
        Box::new(|_cc| Box::new(app)),
    )
    .unwrap();
}

struct NativeApp {
    settings: Arc<Mutex<ServerSettings>>,
    tx: mpsc::Sender<ServerSettings>,
}

impl eframe::App for NativeApp {
    fn update(&mut self, ctx: &eframe::egui::Context, frame: &mut eframe::Frame) {
        egui::CentralPanel::default().show(ctx, |ui| {
            let mut settings = self.settings.lock().unwrap();

            ui.heading("Server Settings");

            ui.horizontal(|ui| {
                ui.label("Port:");
                if ui.text_edit_singleline(&mut settings.port).changed() {
                    eprintln!("Editing!");
                }
            });

            if ui.button("Apply").clicked() {
                if self.tx.try_send(settings.clone()).is_err() {
                    eprintln!("Failed to send settings update");
                }
            }
        });
    }
}
