#![cfg_attr(not(test), windows_subsystem = "windows")] // not(debug_assertions) would hide the console only on release builds.

use axum_template::engine::Engine;
use minijinja::{path_loader, Environment};
use state::AppState;

use std::env;
use std::sync::{Arc, Mutex};
use tokio::sync::mpsc;

use dotenvy::dotenv;
use vbplay::{example_handler, DeviceSelection, SoundDecoder};

mod handlers;
mod icon;
mod server;
mod soundclips;
mod state;
mod ui;
mod vbplay;

fn main() {
    dotenv().ok();

    #[cfg(target_os = "windows")]
    {
        // if you run this from a windows console, we attach to the parent process.
        use windows::Win32::System::Console::{AttachConsole, ATTACH_PARENT_PROCESS};
        unsafe {
            AttachConsole(ATTACH_PARENT_PROCESS).unwrap_or_default();
        }
    }

    let rt = tokio::runtime::Runtime::new().unwrap();

    let device_pattern = env::var("DEVICE_PATTERN").expect("Need a device pattern");
    let folder = env::var("DIRECTORY").expect("Need a directory in env file.");

    let mut jinja = Environment::new();
    jinja.set_loader(path_loader("templates"));
    let template_engine = Engine::from(jinja);

    let mut audio = vbplay::AudioHandler::new(
        DeviceSelection::FindByPattern(device_pattern),
        SoundDecoder::Detect,
    );

    let event_handler = Box::new(example_handler::PressMouseForward::new());
    audio.on_playback(event_handler);
    let audio = Arc::new(Mutex::new(audio));

    let server_settings = Arc::new(Mutex::new(server::ServerSettings {
        port: "3311".to_string(),
    }));
    let should_run = Arc::new(Mutex::new(true));

    let app_state = AppState {
        engine: template_engine,
        clips: soundclips::scan_directory_for_clips(&folder, &["mp3", "ogg", "wav", "flac"])
            .expect("No Soundclips found."),
        player: audio.clone(),
        playback: None,
        settings: server_settings.clone(),
        should_run: should_run.clone(),
    };

    let (tx_ui_to_server, rx_ui_to_server) = mpsc::channel(32);
    let (tx_server_to_ui, rx_server_to_ui) = mpsc::channel(32);

    let server_handle = rt.spawn(async move {
        server::run_server(app_state, rx_ui_to_server, tx_server_to_ui).await;
    });

    // Run the UI on the main thread
    ui::run_ui(
        server_settings,
        should_run,
        tx_ui_to_server,
        rx_server_to_ui,
    );

    if let Err(e) = rt.block_on(server_handle) {
        eprintln!("Server task error: {:?}", e);
    } else {
        println!("Server task completed successfully");
    }

    println!("Setting shutdown timeout...");

    rt.shutdown_timeout(tokio::time::Duration::from_secs(5));

    audio.lock().unwrap().exit();
    drop(audio);

    eprintln!("Reached end of program.");
    //std::process::exit(0);
}
